# tests for the correct expansion of preferences (*not* FoswikiFns, which
# should have their own individual testcase)

package VariableTests;
use strict;
use warnings;

use FoswikiFnTestCase();
our @ISA = qw( FoswikiFnTestCase );

use Foswiki();
use Error qw( :try );

sub set_up {
    my $this = shift;

    $this->SUPER::set_up();

    my $query = Unit::Request->new("");
    $query->path_info("/$this->{test_web}/$this->{test_topic}");
    $this->createNewFoswikiSession( 'scum', $query );
    $this->{test_topicObject}->finish() if $this->{test_topicObject};
    ( $this->{test_topicObject} ) =
      Foswiki::Func::readTopic( $this->{test_web}, $this->{test_topic} );

    return;
}

sub new {
    my ( $class, @args ) = @_;

    return $class->SUPER::new( 'Variables', @args );
}

sub test_embeddedExpansions {
    my $this = shift;
    $this->{session}->{prefs}->setSessionPreferences(
        EGGSAMPLE => 'Egg sample',
        A         => 'EGG',
        B         => 'SAMPLE',
        C         => '%%A%',
        D         => '%B%%',
        E         => '%EGG',
        F         => 'SAMPLE%',
        PA        => 'A',
        SB        => 'B',
        EXEMPLAR  => 'Exem plar',
        XA        => 'EXEM',
        XB        => 'PLAR',
    );

    my $result = $this->{test_topicObject}->expandMacros("%%A%%B%%");
    $this->assert_equals( 'Egg sample', $result );

    $result = $this->{test_topicObject}->expandMacros("%C%%D%");
    $this->assert_equals( 'Egg sample', $result );

    $result = $this->{test_topicObject}->expandMacros("%E%%F%");
    $this->assert_equals( 'Egg sample', $result );

    $result = $this->{test_topicObject}->expandMacros("%%XA{}%%XB{}%%");
    $this->assert_equals( 'Exem plar', $result );

    $result = $this->{test_topicObject}->expandMacros("%%XA%%XB%{}%");
    $this->assert_equals( 'Exem plar', $result );

    $result = $this->{test_topicObject}->expandMacros("%%%PA%%%%SB{}%%%");
    $this->assert_equals( 'Egg sample', $result );

    return;
}

sub test_topicCreationExpansions {
    my $this = shift;

    my $text = <<'END';
%USERNAME%
%STARTSECTION{type="templateonly"}%
Kill me
%ENDSECTION{type="templateonly"}%
%WIKINAME%
%WIKIUSERNAME%
%WEBCOLOR%
%STARTSECTION{name="fred" type="section"}%
%USERINFO%
%USERINFO{format="$emails,$username,$wikiname,$wikiusername"}%
%USER%NOP%INFO{format="$emails,$username,$wikiname,$wikiusername"}%
%ENDSECTION{name="fred" type="section"}%
END
    $this->{test_topicObject}->text($text);
    $this->{test_topicObject}
      ->put( 'PREFERENCE', { name => "BLAH", value => "%WIKINAME%" } );
    $this->{test_topicObject}->expandNewTopic();

    my $xpect = <<"END";
scum

ScumBag
$this->{users_web}.ScumBag
%WEBCOLOR%
%STARTSECTION{name="fred" type="section"}%
scum, $this->{users_web}.ScumBag, scumbag\@example.com
scumbag\@example.com,scum,ScumBag,$this->{users_web}.ScumBag
%USERINFO{format="\$emails,\$username,\$wikiname,\$wikiusername"}%
%ENDSECTION{name="fred" type="section"}%
END
    $this->assert_str_equals( $xpect, $this->{test_topicObject}->text() );
    $this->assert_str_equals( "ScumBag",
        $this->{test_topicObject}->get( 'PREFERENCE', 'BLAH' )->{value} );

    return;
}

sub test_userExpansions {
    my $this = shift;
    $Foswiki::cfg{AntiSpam}{HideUserDetails} = 0;

    my $text = <<'END';
%USERNAME%
%WIKINAME%
%WIKIUSERNAME%
%USERINFO%
%USERINFO{format="$cUID,$emails,$username,$wikiname,$wikiusername"}%
%USERINFO{"WikiGuest" format="$cUID,$emails,$username,$wikiname,$wikiusername"}%
END
    my $result = $this->{test_topicObject}->expandMacros($text);
    my $xpect  = <<"END";
scum
ScumBag
$this->{users_web}.ScumBag
scum, $this->{users_web}.ScumBag, scumbag\@example.com
${Foswiki::Users::TopicUserMapping::FOSWIKI_USER_MAPPING_ID}scum,scumbag\@example.com,scum,ScumBag,$this->{users_web}.ScumBag
$Foswiki::Users::BaseUserMapping::DEFAULT_USER_CUID,,guest,WikiGuest,$this->{users_web}.WikiGuest
END
    $this->annotate( "Foswiki::cfg{Register}{AllowLoginName} == "
          . $Foswiki::cfg{Register}{AllowLoginName} );
    $this->assert_str_equals( $xpect, $result );

    return;
}

sub test_macroParams {
    my $this = shift;

    # Check default on undefined macros
    # Check default given, given but null, not given
    # Check quotes and other standard expansions
    # Check override of standard macros
    $this->{session}->{prefs}->setSessionPreferences(
        ARFLE => '%BARFLE{default="gloop"}%',
        TING  => '%DEFAULT% %DEFAULT{default="tong"}%',
        ALING => '\'%DEFAULT%\' \'%DEFAULT{default="tong"}%\'',
        TOOT  => '\'%NOP%\'',
        WOOF  => '%MIAOW{default="$quot$percent$quot"}%',
        Test  => '"%arg{default="%DEFAULT{default="Y"}%"}%"'
    );
    my $input = <<'INPUT';
| gloop | %BARFLE{default="gloop"}% |
| mong | %ARFLE{BARFLE="mong"}% |
| %DEFAULT% tong | %TING% |
| ding ding | %TING{"ding"}% |
| '' '' | %ALING{""}% |
| 'sweet' | %TOOT{NOP="sweet"}% |
| "%" | %WOOF% |
| p"r"r | %WOOF{MIAOW="p$quot()r$quot()r"}% |
| Test | %Test% |
| Test{"X"} | %Test{"X"}% |
| Test{arg="Z"} | %Test{arg="Z"}% |
INPUT
    my ($topicObject) =
      Foswiki::Func::readTopic( $this->{test_web}, $this->{test_topic} );
    $topicObject->text($input);
    my $result   = $topicObject->expandMacros($input);
    my $expected = <<'EXPECTED';
| gloop | gloop |
| mong | mong |
| %DEFAULT% tong | %DEFAULT% tong |
| ding ding | ding ding |
| '' '' | '' '' |
| 'sweet' | 'sweet' |
| "%" | "%" |
| p"r"r | p"r"r |
| Test | "Y" |
| Test{"X"} | "X" |
| Test{arg="Z"} | "Z" |
EXPECTED
    $this->assert_str_equals( $expected, $result );
    $topicObject->finish();

    return;
}

1;
